
#include <Adafruit_NeoPixel.h>
#include "LEDS_64_panel.h"
#include <PinChangeInt.h>


Adafruit_NeoPixel LEDS = Adafruit_NeoPixel(PIXEL_NUM, LED_PIN, NEO_GRBW + NEO_KHZ800);

uint8_t CharGroup = 0;
volatile bool State = false;

void setup()
{
  pinMode(KEY_PIN, INPUT);
  digitalWrite(KEY_PIN, HIGH);
  PCintPort::attachInterrupt(KEY_PIN, &Key_ISR, FALLING);
    
  ClearMatrix();
  LEDS.begin();
  LEDS.show();
}
//----------------------------------------
void loop()
{
  switch(CharGroup)
  {
    case 0:
      Digits();
    break;

    case 1:
      Figures();
    break;

    case 2:
      Tetris();
    break;

    default:
    break;
  }
}
//----------------------------------------
void Digits()
{
  const uint8_t DIG_NUM = 12;

  const uint8_t *DIGITS[DIG_NUM] = { DIG_0,  DIG_1, DIG_2, DIG_3, DIG_4,
                                     DIG_5,  DIG_6, DIG_7, DIG_8, DIG_9, DIG_10, DIG_11};
  static uint8_t CurDig = 0;
  
  ChangeCharByte(DIGITS[CurDig], MAIN_COLOUR, BACK_COLOUR); //ChangeChar заменить на ChangeCharByte
  LEDS.show();
  delay(SHOW_DELAY);

  CurDig++;
  if (CurDig == DIG_NUM)
    CurDig = 0;

  if (State)
  {
    CharGroup++;
    if (CharGroup == CharGroups)
      CharGroup = 0;

    CurDig = 0;
    State = false;

    PCintPort::detachInterrupt(KEY_PIN);
    delay(300);
    PCintPort::attachInterrupt(KEY_PIN, &Key_ISR, FALLING);
  }
}
//----------------------------------------
void Figures()
{
  const uint8_t FIG_NUM = 11;

  const uint8_t *FIGURES[FIG_NUM] = { SQUARE_1,  SQUARE_2, SQUARE_3, SQUARE_4,
                                     X_Pixel_1,  X_Pixel_2, X_Pixel_3,
                                     X_Ray_1, X_Ray_2, X_Ray_3, X_Ray_1, };
  static uint8_t CurFig = 0;
  
  ChangeChar(FIGURES[CurFig], MAIN_COLOUR, BACK_COLOUR);
  LEDS.show();
  delay(SHOW_DELAY);

  CurFig++;
  if (CurFig == FIG_NUM)
    CurFig = 0;

  if (State)
  {
    CharGroup++;
    if (CharGroup == CharGroups)
      CharGroup = 0;

    CurFig = 0;
    State = false;
    PCintPort::detachInterrupt(KEY_PIN);
    delay(300);
    PCintPort::attachInterrupt(KEY_PIN, &Key_ISR, FALLING);
  }
}
//----------------------------------------
void Tetris()
{
  const uint8_t TET_NUM = 37;

  const uint8_t *TETRIS[TET_NUM] = { Tetris_1, Tetris_2, Tetris_3, Tetris_4,
                                     Tetris_5, Tetris_6, Tetris_7, Tetris_8,
                                     Tetris_9, Tetris_10, Tetris_11, Tetris_12,
                                     Tetris_13, Tetris_14, Tetris_15, Tetris_16,
                                     Tetris_17, Tetris_18, Tetris_19, Tetris_20,
                                     Tetris_21, Tetris_22, Tetris_23, Tetris_24,
                                     Tetris_25, Tetris_26, Tetris_27, Tetris_28,
                                     Tetris_29, Tetris_30, Tetris_31, Tetris_32,
                                     Tetris_33, Tetris_34, Tetris_35, Tetris_36,
                                     Tetris_37,};
  static uint8_t CurTet = 0;
  
  ChangeChar(TETRIS[CurTet], MAIN_COLOUR, BACK_COLOUR);
  LEDS.show();
  delay(TETRIS_DELAY);

  CurTet++;
  if (CurTet == TET_NUM)
    CurTet = 0;

  if (State)
  {
    CharGroup++;
    if (CharGroup == CharGroups)
      CharGroup = 0;

    CurTet = 0;
    State = false;
    PCintPort::detachInterrupt(KEY_PIN);
    delay(300);
    PCintPort::attachInterrupt(KEY_PIN, &Key_ISR, FALLING);
  }
}
//----------------------------------------
void ClearMatrix()
{
  for (uint8_t i = 0; i < PIXEL_NUM; i++)
    LEDS.setPixelColor(i, BACK_COLOUR);
}
//----------------------------------------
void ChangeChar(const uint8_t *CharPixel, uint32_t ForeGround, uint32_t BackGround)
{
  for (uint8_t i = 0; i < STICK_NUM; i++)
  {
    uint8_t StartIndex = i * PIXEL_IN_STICK;

    for (uint8_t j = StartIndex; j < (StartIndex + PIXEL_IN_STICK); j++)
    {
      uint8_t PixelAdr;

      if ((i % 2) == 0)
        PixelAdr = j;
      else
      {
        PixelAdr = StartIndex * 2 + 7 - j;
      }
      
      if (pgm_read_byte(&CharPixel[PixelAdr]) != 0)
        LEDS.setPixelColor(j, ForeGround);
      else
        LEDS.setPixelColor(j, BackGround);
    }
  }
}
//----------------------------------------
void ChangeCharByte(const uint8_t *CharPixel, uint32_t ForeGround, uint32_t BackGround)
{
  for (uint8_t i = 0; i < 8; i++)
  {
    uint8_t RowData = pgm_read_byte(&CharPixel[i]);
    
    for (uint8_t j = 0; j < 8; j++)
    {
      uint8_t PixelData;
      uint8_t PixelNum = i * 8 + j;

      if ((i % 2) == 0)
        PixelData = RowData & (1 << (7 - j));
      else
        PixelData = RowData & (1 << j);

      if (PixelData != 0)
        LEDS.setPixelColor(PixelNum, ForeGround);
      else
        LEDS.setPixelColor(PixelNum, BackGround);
    }
  }
}
//----------------------------------------
void Key_ISR()
{
  State = true;
}




